package fitconnect.impl;

import com.nimbusds.jose.jwk.RSAKey;
import fitconnect.api.services.Sender;
import fitconnect.api.services.auth.OAuthService;
import fitconnect.api.domain.auth.OAuthToken;
import fitconnect.api.services.crypto.CryptoService;
import fitconnect.api.domain.model.metadata.Metadata;
import fitconnect.api.domain.model.metadata.attachment.Attachment;
import fitconnect.api.domain.model.metadata.data.Data;
import fitconnect.api.exceptions.AuthenticationException;
import fitconnect.api.exceptions.EncryptionException;
import fitconnect.impl.logger.SdkLogger;
import fitconnect.api.services.validation.CertificateValidator;
import fitconnect.api.domain.validation.ValidationResult;
import fitconnect.impl.auth.FitCoAuthService;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;


public class SubmissionSender implements Sender {

    private static final Logger logger = SdkLogger.defaultLogger(FitCoAuthService.class);

    private final OAuthService authService;
    private final CertificateValidator certificateValidator;
    private final CryptoService cryptoService;

    public SubmissionSender(final OAuthService authService, final CryptoService encryptionService,
                            final CertificateValidator certificateValidator) {
        this.authService = authService;
        this.cryptoService = encryptionService;
        this.certificateValidator = certificateValidator;
    }


    @Override
    public ValidationResult validatePublicKey(RSAKey publicKey) {
        return certificateValidator.validatePublicKey(publicKey);
    }

    @Override
    public Optional<Data> encryptSubmissionData(RSAKey publicKey, Data data) {
        try {
            // TODO re-hash and return encrypted data
            cryptoService.encryptString(publicKey, data.getHash().getContent());
            return Optional.of(data);
        } catch (EncryptionException e) {
            logger.log(Level.SEVERE, "Data could not be encrypted");
            return Optional.empty();
        }
    }

    @Override
    public Optional<Attachment> encryptAttachment(RSAKey publicKey, Attachment attachment) {
        final byte[] attachmentData = attachment.getHash().getContent().getBytes(StandardCharsets.UTF_8);
        try {
            final String encryptedAttachment = cryptoService.encryptBytes(publicKey, attachmentData);
            // TODO re-hash and return encrypted attachment
            return Optional.of(attachment);
        } catch (EncryptionException e) {
            logger.log(Level.SEVERE, "Attachment could not be encrypted");
            return Optional.empty();
        }
    }

    @Override
    public Optional<OAuthToken> retrieveOAuthToken(String clientId, String clientSecret, String... scope) {
        try {
            return Optional.of(authService.authenticate(clientId, clientSecret, scope));
        } catch (AuthenticationException e) {
            return Optional.empty();
        }
    }

    @Override
    public Optional<Metadata> createMetadata(Data data, List<Attachment> attachments) {
        return Optional.empty();
    }
}
