package fitconnect.impl;

import com.nimbusds.jose.jwk.RSAKey;
import fitconnect.api.services.Subscriber;
import fitconnect.api.services.auth.OAuthService;
import fitconnect.api.domain.auth.OAuthToken;
import fitconnect.api.services.crypto.CryptoService;
import fitconnect.api.domain.model.metadata.Metadata;
import fitconnect.api.domain.model.metadata.attachment.Attachment;
import fitconnect.api.domain.model.metadata.data.Data;
import fitconnect.api.exceptions.AuthenticationException;
import fitconnect.api.exceptions.DecryptionException;
import fitconnect.impl.logger.SdkLogger;
import fitconnect.api.services.validation.MetadataValidator;
import fitconnect.api.domain.validation.ValidationResult;
import fitconnect.impl.auth.FitCoAuthService;

import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;


public class SubmissionSubscriber implements Subscriber {

    private static final Logger logger = SdkLogger.defaultLogger(FitCoAuthService.class);

    private final OAuthService authService;
    private final CryptoService cryptoService;
    private final MetadataValidator metadataValidator;

    public SubmissionSubscriber(final OAuthService authService, final CryptoService cryptoService,
                                MetadataValidator metadataValidator){
        this.authService = authService;
        this.cryptoService = cryptoService;
        this.metadataValidator = metadataValidator;
    }

    @Override
    public Optional<OAuthToken> retrieveOAuthToken(String clientId, String clientSecret, String... scope) {
        try {
            return Optional.of(authService.authenticate(clientId, clientSecret, scope));
        } catch (AuthenticationException e) {
            logger.log(Level.SEVERE, "Retrieving the OAuth token failed: ", e.getMessage());
            return Optional.empty();
        }
    }

    @Override
    public Optional<Data> decryptSubmissionData(RSAKey privateKey, Data encryptedData) {
        try {
            cryptoService.decryptString(privateKey, encryptedData.getHash().getContent());
            // TODO create and rehash decrypted data
            return Optional.of(encryptedData);
        } catch (DecryptionException e) {
            logger.log(Level.SEVERE, "Submission data could not be decrypted: ", e.getMessage());
            return Optional.empty();
        }
    }

    @Override
    public Optional<Attachment> decryptAttachment(RSAKey privateKey, Attachment encryptedAttachment) {
        try {
            cryptoService.decryptBytes(privateKey, encryptedAttachment.getHash().getContent());
            // TODO create and rehash decrypted data
            return Optional.of(encryptedAttachment);
        } catch (DecryptionException e) {
            logger.log(Level.SEVERE, "Submission data could not be decrypted: ", e.getMessage());
            return Optional.empty();
        }
    }

    @Override
    public ValidationResult validateMetadataSchema(Metadata metadata, String jsonSchema) {
        return metadataValidator.validateMetadataSchema(metadata, jsonSchema);
    }

    @Override
    public ValidationResult validateMetadataHashValues(Metadata metadata) {
        return metadataValidator.validateMetadataHashValues(metadata);
    }
}
